codeunit 50014 DynamicwebOrdersSubscriber
{
    EventSubscriberInstance = StaticAutomatic;

    var
        XmlHelper: Codeunit XmlHelper;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnBeforeExecutePutOrdersRequest', '', true, true)]
    procedure OnBeforeExecutePutOrdersRequest(var requestDoc: XmlDocument; var stopExecution: Boolean; var responseDoc: XmlDocument);
    begin
        XmlHelper.AddCustomElement('OnBeforeExecutePutOrdersRequest', responseDoc);
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnAfterExecutePutOrdersRequest', '', true, true)]
    procedure OnAfterExecutePutOrdersRequest(var requestDoc: XmlDocument; var responseDoc: XmlDocument);
    begin
        XmlHelper.AddCustomElement('OnAfterExecutePutOrdersRequest', responseDoc);
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnBeforeWriteNote', '', true, true)]
    procedure OnBeforeWriteNote(var noteText: Text; var skipWrittingNote: Boolean; salesHeader: Record "Sales Header");
    begin
        noteText := 'Custom note text';
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnBeforeCalculateOrderRequest', '', true, true)]
    procedure OnBeforeCalculateOrderRequest(var orderNode: XmlNode; var stopExecution: Boolean; var responseRootNode: XmlNode);
    begin
        XmlHelper.AddCustomElement('OnBeforeCalculateOrderRequest', responseRootNode);
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnAfterCalculateOrderRequest', '', true, true)]
    procedure OnAfterCalculateOrderRequest(var orderNode: XmlNode; var responseRootNode: XmlNode);
    begin
        XmlHelper.AddCustomElement('OnAfterCalculateOrderRequest', responseRootNode);
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnBeforeCreateOrderRequest', '', true, true)]
    procedure OnBeforeCreateOrderRequest(var orderNode: XmlNode; var stopExecution: Boolean; var responseRootNode: XmlNode);
    begin
        XmlHelper.AddCustomElement('OnBeforeCreateOrderRequest', responseRootNode);
    end;


    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnAfterCreateOrderRequest', '', true, true)]
    procedure OnAfterCreateOrderRequest(var orderNode: XmlNode; var responseRootNode: XmlNode);
    begin
        XmlHelper.AddCustomElement('OnAfterCreateOrderRequest', responseRootNode);
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnBeforeUpdateSalesHeader', '', true, true)]
    procedure OnBeforeUpdateSalesHeader(var salesheader: Record "Sales Header"; orderRequestNode: XmlNode);
    var
        node: XmlNode;
        comment: OutStream;
    begin
        orderRequestNode.SelectSingleNode('/tables/table[@tableName=''EcomOrders'']/item/column[@columnName=''OrderId'']', node);
        salesheader."External Document No." := node.AsXmlElement().InnerText;
        orderRequestNode.SelectSingleNode('/tables/table[@tableName=''EcomOrders'']/item/column[@columnName=''YourReference'']', node);
        salesheader."Your Reference" := node.AsXmlElement().InnerText;
        orderRequestNode.SelectSingleNode('/tables/table[@tableName=''EcomOrders'']/item/column[@columnName=''OrderSalesPersonCode'']', node);
        salesheader."Salesperson Code" := node.AsXmlElement().InnerText;
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnBeforeCreateSalesLine', '', true, true)]
    procedure OnBeforeCreateSalesLine(var salesline: Record "Sales Line"; var salesheader: Record "Sales Header"; requestDoc: XmlDocument; var requestOrderNode: XmlNode; var requestOrderLineNode: XmlNode);
    begin
        salesline.Description := 'extendable from OnBeforeCreateSalesLine';
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnBeforeAddSalesLineToResponse', '', true, true)]
    procedure OnBeforeAddSalesLineToResponse(var salesline: Record "Sales Line"; var salesheader: Record "Sales Header"; var dynamicwebDiscountId: Text; var dynamicwebDiscountName: Text; var cancelAdd: boolean);
    begin
        //Skip adding sales line with 1936-S product to response xml
        if (salesLine."No." = '1936-S') then
            cancelAdd := true;
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnAfterAddSalesLineToResponse', '', true, true)]
    procedure OnAfterAddSalesLineToResponse(var responseOrderLineXml: XmlNode; salesline: Record "Sales Line"; salesheader: Record "Sales Header");
    begin
        XmlHelper.AddField(responseOrderLineXml, 'OnAfterAddSalesLineToResponse', 'extendable');
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnAfterAddBCOrderDiscountOrderLineToResponse', '', true, true)]
    procedure OnAfterAddBCOrderDiscountOrderLineToResponse(var responseOrderLineXml: XmlNode; salesheader: Record "Sales Header");
    begin
        XmlHelper.AddField(responseOrderLineXml, 'OnAfterAddBCOrderDiscountOrderLineToResponse', 'extendable');
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnBeforeInsertDynamicwebShippingSalesLine', '', true, true)]
    procedure OnBeforeInsertDynamicwebShippingSalesLine(var salesline: Record "Sales Line"; var cancelAdd: Boolean);
    begin
        salesline."Description" := 'extendable from OnBeforeInsertDynamicwebShippingSalesLine';
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnAfterAddSalesHeaderToResponse', '', true, true)]
    procedure OnAfterAddSalesHeaderToResponse(var responseSalesHeaderXml: XmlNode; salesheader: Record "Sales Header");
    begin
        XmlHelper.AddField(responseSalesHeaderXml, 'OnAfterAddSalesHeaderToResponse', 'extendable');
    end;

    /// <summary>
    /// Occurs before order is created
    /// </summary>
    /// <param name="XMLOrderNode"></param>
    /// <param name="customer"></param>
    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnBeforeCreateOrder', '', true, true)]
    procedure OnBeforeCreateOrder(XMLOrderNode: XmlNode; var customer: record Customer);
    begin
        XmlHelper.AddField(XMLOrderNode, 'OnBeforeCreateOrder', 'extendable');
    end;

    /// <summary>
    /// Occurs after order is created
    /// </summary>
    /// <param name="salesheader"></param>
    /// <param name="XMLOrderNode"></param>
    /// <param name="customer"></param>
    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnAfterCreateOrder', '', true, true)]
    procedure OnAfterCreateOrder(var salesheader: record "Sales Header"; XMLOrderNode: XmlNode; var customer: record Customer);
    begin
        XmlHelper.AddField(XMLOrderNode, 'OnAfterCreateOrder', 'extendable');
    end;

    /// <summary>
    /// Occurs in calculate order request before order lines are processed
    /// and after the sales header was set
    /// </summary>
    /// <param name="salesheader"></param>
    /// <param name="XMLOrderNode"></param>
    /// <param name="customer"></param>
    /// <param name="requestDoc"></param>
    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'CalculateOrderOnBeforeSetSalesLines', '', true, true)]
    procedure CalculateOrderOnBeforeSetSalesLines(var salesheader: record "Sales Header"; XMLOrderNode: XmlNode; var customer: record Customer; var requestDoc: XmlDocument)
    begin
        XmlHelper.AddField(XMLOrderNode, 'CalculateOrderOnBeforeSetSalesLines', 'extendable');
    end;

    /// <summary>
    /// Occurs after calculate order method is processed    
    /// </summary>
    /// <param name="salesheader"></param>
    /// <param name="XMLOrderNode"></param>
    /// <param name="customer"></param>
    [EventSubscriber(ObjectType::Codeunit, Codeunit::DynamicwebOrdersPublisher, 'OnAfterCalculateOrder', '', true, true)]
    procedure OnAfterCalculateOrder(var salesheader: record "Sales Header"; XMLOrderNode: XmlNode; var customer: record Customer)
    begin
        XmlHelper.AddField(XMLOrderNode, 'OnAfterCalculateOrder', 'extendable');
    end;
}